#!/usr/bin/env python
# coding:utf-8
""" 
@author: nivic ybyang7
@license: Apache Licence 
@file: create.py
@time: 2022/07/26
@contact: ybyang7@iflytek.com
@site:  
@software: PyCharm
管理配置中心的
"""

def create_config(name):
    print("creating project")