# {{vars.project_name}}

* This is your first aiges python wrapper project: {{vars.project_name}}, located at: {{vars.project_path}}

## Debug

to debug or run your wrapper local, please implement .... todo
# {{vars.test}}